import inquirer from "inquirer";
import { tsquery } from "@phenomnomnominal/tsquery";
import chalk from "chalk";
import path, {
  VIEW_URL,
  SRC_URL,
  removeSlash,
  TEMPLATE_URL
} from "../utils/path.mjs";
import { humpToBar } from "../utils/name.mjs";
import ejs from "ejs";
import {
  createFile,
  changeFile,
  getFileCon,
  fileExist,
  createDir,
} from "../utils/file.mjs";

export const createComponent = async () => {
  const { type } = await inquirer.prompt([
    {
      type: "list",
      name: "type",
      message: "请选择您想创建的组件类型",
      choices: [
        {
          name: "通用组件（src/components中）",
          value: "base",
        },
        {
          name: "普通组件（src/views中）",
          value: "common",
        },
      ],
    },
  ]);

  const { componentName } = await inquirer.prompt([
    {
      type: "String",
      name: "componentName",
      message: "请填写页面组件名称[首字母大写，仅支持字母]",
      validate: (v) => {
        const reg = /^[A-Z]\w*$/;

        if (reg.test(v)) {
          return true;
        } else {
          console.warn(chalk.red("组件名称首字母大写，仅支持字母"));
          return false;
        }
      },
    },
  ]);

  const isBase = type === "base";
  // 相对路径，/src下
  let filePath = "components";

  if (!isBase) {
    const { path } = await inquirer.prompt([
      {
        type: "String",
        name: "path",
        message: "请填写组件代码文件路径[src/views下的相对路径]",
        validate: async (v) => {
          // 校验：当输入的值为是string类型，才能按回车，否则按了回车并无效果
          const filePath = VIEW_URL + removeSlash(v);

          if (!v) {
            console.warn(chalk.red("请填写组件代码文件路径"));
            return false;
          } else if (!isBase && !fileExist(filePath)) {
            console.warn(chalk.red(`\n路径${filePath}不存在`));
            return false;
          } else {
            return true;
          }
        },
      },
    ]);

    filePath = removeSlash(path);
  }

  createComponentFile(isBase, filePath, componentName);
};

const createComponentFile = async (
  isBase,
  filePath,
  componentName
) => {
  const fileName = humpToBar(componentName);

  // 创建代码
  const className = filePath.replace("/", "-") + "-" + fileName;

  const componentCode = await ejs.renderFile(TEMPLATE_URL + "/component.ejs", {
    className,
    componentName,
  });

  // 创建文件
  if (isBase) {
    // 创建组件文件
    const componetFilePath = path.join(SRC_URL, `/components/${fileName}`);

    createDir(componetFilePath);

    createFile({
      path: componetFilePath,
      name: "index.tsx",
      con: componentCode,
    });

    // 修改配置
    const configPath = path.join(SRC_URL, "/components/index.ts");
    const configCode = getFileCon(configPath);

    const resCode = tsquery.replace(configCode, "EndOfFileToken", () => {
      return `export { ${componentName} } from "./${fileName}";
`;
    });

    changeFile({
      url: configPath,
      con: resCode,
    });
  } else {
    const rootPath = path.join(VIEW_URL, filePath);

    createFile({
      path: rootPath,
      name: `${fileName}.tsx`,
      con: componentCode,
    });
  }
};
