import inquirer from "inquirer";
import { tsquery } from "@phenomnomnominal/tsquery";
import chalk from "chalk";
import path, { MODEL_URL, removeSlash, TEMPLATE_URL } from "../utils/path.mjs";
import { humpToBar } from "../utils/name.mjs";
import ejs from "ejs";
import {
  createFile,
  changeFile,
  getFileCon,
  fileExist,
  createDir,
} from "../utils/file.mjs";

export const createModel = async () => {
  const { parentPath } = await inquirer.prompt([
    {
      type: "String",
      name: "parentPath",
      message:
        "请填写model的路径[src/models下的相对路径，小写字母、仅支持一级]",
      validate: (v) => {
        const reg = /^[a-z/][a-z-]*[a-z/]$/;
        // 校验：当输入的值为是string类型，才能按回车，否则按了回车并无效果
        if (reg.test(v)) {
          return true;
        } else {
          console.warn(chalk.red("model的路径小写字母、仅支持一级"));
          return false;
        }
      },
    },
  ]);

  const { modelName } = await inquirer.prompt([
    {
      type: "String",
      name: "modelName",
      message: "请填写model的名称（驼峰）",
      validate: (v) => {
        const reg = /^\w*$/;
        // 校验：当输入的值为是string类型，才能按回车，否则按了回车并无效果
        if (reg.test(v)) {
          return true;
        } else {
          console.warn(chalk.red("model名称仅支持字母"));
          return false;
        }
      },
    },
  ]);

  createModelFile(removeSlash(parentPath), modelName);
};

export const createModelFile = async (parentPath, modelName) => {
  const modeFilePath = path.join(MODEL_URL, parentPath);
  const modelFileName = humpToBar(modelName);
  const isFileExist = fileExist(modeFilePath);
  const modelKey =
    parentPath.replace(/-\w/g, (str) => str.replace("-", "").toUpperCase()) +
    modelName.replace(/^\w/, (str) => str.toUpperCase());

  console.log(modelKey);

  if (!isFileExist) {
    createDir(modeFilePath);
  }

  const modelCode = await ejs.renderFile(TEMPLATE_URL + "/model.ejs", {
    modelName: modelKey,
  });

  createFile({
    path: modeFilePath,
    name: modelFileName + ".ts",
    con: modelCode,
  });

  const modeConfigPath = path.join(MODEL_URL, "index.ts");
  const oldCode = getFileCon(modeConfigPath);

  // 添加import
  const importSelect = "ImportDeclaration";
  const importLength = tsquery(oldCode, importSelect).length;
  let currImport = 0;

  const codeAfterAddImport = tsquery.replace(oldCode, importSelect, (node) => {
    currImport += 1;

    if (importLength === currImport) {
      return `${node.getText()}
import { ${modelKey} } from "./${parentPath}/${modelFileName}";`;
    } else {
      return node.getText();
    }
  });

  // 添加配置项
  const codeAfterAddType = tsquery.replace(
    codeAfterAddImport,
    "InterfaceDeclaration:has(Identifier[name=RootModel]) > PropertySignature:last-child",
    (node) => {
      return `${node.getText()}
  ${modelKey}: typeof ${modelKey};`;
    }
  );

  const finalStr = tsquery.replace(
    codeAfterAddType,
    "VariableStatement:has(Identifier[name=RootModel]) ObjectLiteralExpression > ShorthandPropertyAssignment:last-child",
    (node) => {
      return `${node.getText()},
  ${modelKey}`;
    }
  );

  changeFile({
    url: modeConfigPath,
    con: finalStr,
  });
};
